#include <XSUtil/Utils/XSutility.h>
#include <cfortran.h>

// Functions called FROM here to Fortran:

PROTOCCALLSFSUB17(NONEQR,noneqr,FLOATV,FLOATV,INT,INT,INT,INT,DOUBLEV,DOUBLEV,DOUBLEV,
        DOUBLEV,DOUBLEV,DOUBLEV,DOUBLEV,INTV,FLOATV,INTV,INTV)
#define NONEQR(tempr,tau,n,nzmax,nzpmax,nionp,vr,vl,eig,feqs,fs,fspec,work,lt,fout,ionel,ionstage) \
           CCALLSFSUB17(NONEQR,noneqr,FLOATV,FLOATV,INT,INT,INT,INT,DOUBLEV,DOUBLEV,DOUBLEV, \
             DOUBLEV,DOUBLEV,DOUBLEV,DOUBLEV,INTV,FLOATV,INTV,INTV, \
             tempr,tau,n,nzmax,nzpmax,nionp,vr,vl,eig,feqs,fs,fspec,work,lt,fout,ionel,ionstage)

// Functions called from Fortran TO here:

void ionsneqr(float* tmp, float* tau, int n, int nzmax, int nionp,
                float* fout, int* ionel, int* ionstage);
FCALLSCSUB8(ionsneqr,IONSNEQR,ionsneqr,FLOATV,FLOATV,INT,INT,INT,FLOATV,INTV,INTV)

void ionsneqr(float* tmp, float* tau, int n, int nzmax, int nionp,
                float* fout, int* ionel, int* ionstage)
{

//
//        Calculates ionization fractions fout at electron temperature
//        tmp(n) and ionization parameter tau(n), for electron 
//        temperatures tmp given in a tabular form as a function of 
//        ionization parameter tau. 
//
//        Input:  tmp      - temperatures (K)
//                tau      - ionization timescales  (cm^-3 s)
//                n        - number of ionization timescales
//                nzmax    - max number of ions tabulated for any element
//                nionp    - number of ions
//        Output: fout     - ionic concentrations
//                ionel    - the element for each fout entry
//                ionstage - the ion stage for each fout entry
//                           (ionel+1 = fully stripped)
//

   using namespace XSutility;
   
   // Grab memory for the temporary arrays required by NONEQR.

   auto_array_ptr<double> apVr(new double[nzmax*nzmax]); 
   auto_array_ptr<double> apVl(new double[nzmax*nzmax]); 
   auto_array_ptr<double> apEig(new double[nzmax]); 
   auto_array_ptr<double> apFeqs(new double[nzmax+1]); 
   auto_array_ptr<double> apFs(new double[nzmax+1]); 
   auto_array_ptr<double> apFspec(new double[nzmax]); 
   auto_array_ptr<double> apWork(new double[nzmax]); 
   auto_array_ptr<int> apLt(new int[n]);
   
   for (int ion=0; ion<nionp; ++ion)
      fout[ion] = 0.0;

   int nzpmax = nzmax+1;      
   NONEQR(tmp, tau, n, nzmax, nzpmax, nionp, apVr.get(), apVl.get(),
                apEig.get(), apFeqs.get(), apFs.get(), apFspec.get(),
                apWork.get(), apLt.get(), fout, ionel, ionstage); 
}
